'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 * Detection Object with properties to aggregate the data 
 * from a detection in Deepsight Toolkit
 */

/**
 * Creates a RawDetection  Object
 * @constructor
 * @param {Number} id 
 */
function RawDetection(id) {

    this.MAX_VALID_AGE_GENDER_YAW_ANGLE = 30;
    this.MIN_VALID_BBOX_SIZE            = 20;
    this.MAX_VALID_IMPRESSION_YAW_ANGLE = 45;
    this.MAX_VALID_FACE_MASK_YAW_ANGLE  = 50;

    this.id = id;
    this.ticks = 0;
    this.ageGenderTicks = 0;
    this.rawAge = 0;
    this.rawGender = 0;
    this.rawSmile = 0;
    this.aggregatedAge = 0;
    this.aggregatedGender = 0;
    this.aggregatedSmile = 0;
    this.maxSmileViewing = 0;
    this.maxSmile = 0;
    this.smile = 0;
    this.age = 0;
    this.gender = undefined;
    this.smile = 0;
    this.viewing = 0;
    this.impression = 0;

    this.detectionTime = 0;
    this.detectionDuration = 0;
    this.dwellDuration = 0;
    this.totalAttentionDuration = 0;
    this.currentAttentionDuration = 0;

    this.position = {};
    this.position.x = -1;
    this.position.y = -1;
    this.position.w = 0;
    this.position.h = 0;

    this.rotation = {};
    this.rotation.yaw = 0;
    this.rotation.pitch = 0;
    this.rotation.roll = 0;

    this.gaze = {};
    this.gaze.x = 0;
    this.gaze.y = 0;

    this.distance = 0.0;

    this.faceMask = 0;
    this.last_timestamp = undefined;
}


RawDetection.prototype.update = function (detection, timestamp) {

    try {
        ++this.ticks;

        this.rawAge = detection.age;
        this.rawGender = detection.gender;
        this.rawSmile = detection.smile;

        if (this.isAgeGenderInThreshold(detection)) {

            ++this.ageGenderTicks;
            this.aggregatedAge += parseInt(detection.age);
            this.aggregatedGender += parseInt(detection.gender);
            this.age = Math.round(this.aggregatedAge / this.ageGenderTicks);
            this.gender = Math.round(this.aggregatedGender / this.ageGenderTicks);
            
        }

        this.aggregatedSmile += parseInt(detection.smile);

        this.smile = Math.round(this.aggregatedSmile / this.ticks);
        this.setMaxSmiles(detection);

        this.viewing = (detection.viewing) ? 1 : 0;
        this.impression = this.isImpression(detection);

        this.detectionTime = detection.detection_time;
        this.detectionDuration = parseInt(detection.detection_duration);
        this.dwellDuration    += this.getDwellDuration(detection);
        this.totalAttentionDuration = parseInt(detection.total_attention_duration);
        this.currentAttentionDuration = parseInt(detection.current_attention_duration);

        this.position.x = parseInt(detection.position.x);
        this.position.y = parseInt(detection.position.y);
        this.position.w = parseInt(detection.position.w);
        this.position.h = parseInt(detection.position.h);

        this.rotation.yaw = parseInt(detection.rotation.yaw);
        this.rotation.pitch = parseInt(detection.rotation.pitch);
        this.rotation.roll = parseInt(detection.rotation.roll);

        this.gaze.x = parseInt(detection.gaze.x);
        this.gaze.y = parseInt(detection.gaze.y);

        this.distance = parseFloat(detection.distance_from_camera);

        this.faceMask = detection.face_mask;

        this.last_timestamp = timestamp;

    } catch (error) {

        console.log("RawDetection.update Error : " + error);

    }
}

RawDetection.prototype.setMaxSmiles = function (detection) {
    if (parseInt(detection.smile) > this.maxSmile) { this.maxSmile = parseInt(detection.smile) };
    if (parseInt(detection.viewing) && parseInt(detection.smile) > this.maxSmileViewing) { this.maxSmileViewing = parseInt(detection.smile) };
}

RawDetection.prototype.isBBOXInThreshold = function (detection) {
    return (Math.min(parseInt(detection.position.w), parseInt(detection.position.h)) >= this.MIN_VALID_BBOX_SIZE);
}

RawDetection.prototype.isAgeGenderInThreshold = function (detection) {
    return (Math.abs(parseInt(detection.rotation.yaw)) <= this.MAX_VALID_AGE_GENDER_YAW_ANGLE && this.isBBOXInThreshold(detection));
}

RawDetection.prototype.isMaskDetectorInThreshold = function (detection) {
    return (Math.abs(parseInt(detection.rotation.yaw)) <= this.MAX_VALID_FACE_MASK_YAW_ANGLE && this.isBBOXInThreshold(detection));
}

RawDetection.prototype.isImpression = function (detection) {
    return (Math.abs(parseInt(detection.rotation.yaw)) <= this.MAX_VALID_IMPRESSION_YAW_ANGLE);
}


RawDetection.prototype.getDwellDuration = function( detection ) {

    if(detection.current_attention_duration === 0) {
        if(this.isImpression(detection))
            return (detection.detection_duration - this.detectionDuration);
        else return 0;
    } else {
        return (detection.total_attention_duration - this.totalAttentionDuration);
    }

}

RawDetection.prototype.parseGenderToHuminizedValue = function () { return ((this.gender < 0) ? 'Male' : (this.gender > 0) ? 'Female' : 'Uncertain'); }

RawDetection.prototype.parseMillisecondsToSeconds = function (value) {
    return (value > 0) ? (value / 1000) : 0.000;
}