'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 * Creates an Raw Session Aggregated Data Object with totals and avg properties
 *  updated when _computeData is called
 * 
 * @constructor
 *
 */
function RawSessionAggregatedData() {
    
    this.count                    = 0;
    this.maleCount                = 0;
    this.malePercentage           = 0;
    this.femaleCount              = 0;
    this.femalePercentage         = 0;
    this.uncertainCount           = 0;
    this.uncertainPercentage      = 0;
    this.impressionsCount         = 0;
    this.viewersCount             = 0;
    this.age                      = 0;
    this.smileViewing             = 0;
    this.dwellDuration            = 0;
    this.attentionDuration        = 0;
    this.maleDwellDuration        = 0;
    this.femaleDwellDuration      = 0;
    this.maleAttentionDuration    = 0;
    this.femaleAttentionDuration  = 0;
    this.withMaskCount            = 0;
    this.withoutMaskCount         = 0;
}

RawSessionAggregatedData.prototype._computeData = function( rawSessionDetections ) {
    
    try {
        
        this._resetRawSessionAggregatedData();

        var count_people = rawSessionDetections.size;
        var _uncertainGender, _totalAge, _totalSmile = 0;
        _uncertainGender =  _totalAge = _totalSmile = 0;
        var self = this;

        if (count_people) {
            rawSessionDetections.forEach(function(detection, key){
                if(detection.gender < 0){
                    self.maleCount++;
                    self.maleDwellDuration       += detection.dwellDuration;
                    self.maleAttentionDuration   += detection.attentionDuration;
                } else if(detection.gender > 0){
                    self.femaleCount++;
                    self.femaleDwellDuration     += detection.dwellDuration;
                    self.femaleAttentionDuration += detection.attentionDuration;
                } else {
                    self.uncertainCount++;
                }

                _totalAge   += detection.age;
                _totalSmile += detection.maxSmileViewing;
                self.detectionDuration += detection.detectionDuration;
                self.dwellDuration += detection.dwellDuration;
                self.attentionDuration += detection.attentionDuration;
               
                if(detection.impression) self.impressionsCount++;
                if(detection.viewer)     self.viewersCount++;

                if(detection.faceMask) self.withMaskCount++; else self.withoutMaskCount++;
            });

            //Compute counter, avg and percentage 
            this.count = count_people;
            this.malePercentage = parseInt(Math.floor((self.maleCount/count_people)*100).toFixed(0));
            this.femalePercentage = parseInt(Math.floor((self.femaleCount/count_people)*100).toFixed(0));
            this.uncertainPercentage = parseInt(Math.floor((self.uncertainCount/count_people)*100).toFixed(0));
            this.age = parseInt((_totalAge/count_people).toFixed(0));
            this.smileViewing = parseInt((_totalSmile/count_people).toFixed(0));
            this.detectionDuration = this.parseMillisecondsToSeconds(this.detectionDuration);
            this.dwellDuration = this.parseMillisecondsToSeconds(this.dwellDuration);
            this.attentionDuration = this.parseMillisecondsToSeconds(this.attentionDuration);
            this.maleDwellDuration = this.parseMillisecondsToSeconds(this.maleDwellDuration);
            this.femaleDwellDuration = this.parseMillisecondsToSeconds(this.femaleDwellDuration);
            this.maleAttentionDuration = this.parseMillisecondsToSeconds(this.maleAttentionDuration);
            this.femaleAttentionDuration = this.parseMillisecondsToSeconds(this.femaleAttentionDuration);

        }

    } catch (error) {
        console.log("RawSessionAggregatedData._computeData Error : " + error);
    }
}


RawSessionAggregatedData.prototype._resetRawSessionAggregatedData = function( data ) {
    this.count                    = 0;
    this.maleCount                = 0;
    this.malePercentage           = 0;
    this.femaleCount              = 0;
    this.femalePercentage         = 0;
    this.uncertainCount           = 0;
    this.uncertainPercentage      = 0;
    this.impressionsCount         = 0;
    this.viewersCount             = 0;
    this.age                      = 0;
    this.smileViewing             = 0;
    this.detectionDuration        = 0;
    this.dwellDuration            = 0;
    this.attentionDuration        = 0;
    this.maleDwellDuration        = 0;
    this.femaleDwellDuration      = 0;
    this.maleAttentionDuration    = 0;
    this.femaleAttentionDuration  = 0;
    this.withMaskCount            = 0;
    this.withoutMaskCount         = 0;
}


RawSessionAggregatedData.prototype.parseMillisecondsToSeconds = function (value) {
    return (value > 0) ? (value / 1000) : 0.000;
}