'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 * Detection Object with properties to aggregate the data 
 * from a detection in Deepsight Toolkit
 */

/**
 * Creates a RealTimeDetection  Object
 * @constructor
 * @param {Number} id 
 */
function RawSessionDetection(id) {
    
    this.MAX_VALID_AGE_GENDER_YAW_ANGLE = 30;
    this.MIN_VALID_BBOX_SIZE            = 20;
    this.MAX_VALID_IMPRESSION_YAW_ANGLE = 45;
    this.MAX_VALID_FACE_MASK_YAW_ANGLE  = 50;
    
    this.id = id;
    this.ageGenderTicks = 0;
    this.aggregatedAge = 0;
    this.age = 0;
    this.aggregatedGender = 0;
    this.gender = 0;
    this.aggregatedSmile = 0;
    this.ticks = 0;
    this.smile = 0;
    this.maxSmileViewing = 0;
    this.maxSmile = 0;

    this.viewer = 0;
    this.impression = 0;

    this.initialDetectionDuration = undefined;
    this.detectionDuration = 0;
    this.initialDwellDuration = undefined;
    this.dwellDuration = 0;
    this.initialAttentionDuration = undefined;
    this.attentionDuration = 0;


    this.faceMask = 0;
}


RawSessionDetection.prototype.update = function ( detection ,  userMaskThreshold ) {

    try {

        if (this.isAgeGenderInThreshold(detection)) {
            ++this.ageGenderTicks;
            this.aggregatedAge += parseInt(detection.rawAge);
            this.aggregatedGender += parseInt(detection.rawGender);
            this.age = Math.round(this.aggregatedAge / this.ageGenderTicks);
            this.gender = Math.round(this.aggregatedGender / this.ageGenderTicks);
        }

        ++this.ticks;
        this.aggregatedSmile += parseInt(detection.rawSmile);
        if (this.aggregatedSmile > 0) this.smile = parseInt(detection.aggregatedSmile / this.ticks);
        this.setMaxSmiles(detection)

        if (!this.viewing && detection.viewing) this.viewer = 1;

        if (!this.impression && this.isImpression(detection)) this.impression = 1;

        this.setInitialDurations(detection);
        this.dwellDuration += this.getDwellDuration(detection);
        this.detectionDuration = detection.detectionDuration - this.initialDetectionDuration;
        this.attentionDuration = detection.totalAttentionDuration - this.initialAttentionDuration;
        
        if(this.isMaskDetectorInThreshold( detection ) && (detection.faceMask > userMaskThreshold))
                this.faceMask = 1;

    } catch (error) {

        console.log("RawSessionDetection.update Error : " + error);

    }
}

RawSessionDetection.prototype.finalize = function () {
    if (this.dwellDuration < this.attentionDuration) this.dwellDuration = this.attentionDuration;

    this.dwellDuration     = this.parseMillisecondsToSeconds(this.dwellDuration);
    this.detectionDuration = this.parseMillisecondsToSeconds(this.detectionDuration);
    this.attentionDuration = this.parseMillisecondsToSeconds(this.attentionDuration);

    this.viewer            = this.parseBooleanToHuminizedValue(this.viewer);
    this.impression        = this.parseBooleanToHuminizedValue(this.impression);
    this.faceMask          = this.parseBooleanToHuminizedValue(this.faceMask);
}

RawSessionDetection.prototype.isBBOXInThreshold = function (detection) {
    return (Math.min(parseInt(detection.position.w), parseInt(detection.position.h)) >= this.MIN_VALID_BBOX_SIZE);
}

RawSessionDetection.prototype.isAgeGenderInThreshold = function (detection) {
    return (Math.abs(parseInt(detection.rotation.yaw)) <= this.MAX_VALID_AGE_GENDER_YAW_ANGLE && this.isBBOXInThreshold(detection));
}

RawSessionDetection.prototype.isMaskDetectorInThreshold = function (detection) {
    return (Math.abs(parseInt(detection.rotation.yaw)) <= this.MAX_VALID_FACE_MASK_YAW_ANGLE && this.isBBOXInThreshold(detection));
}

RawSessionDetection.prototype.isImpression = function (detection) {
    return (Math.abs(parseInt(detection.rotation.yaw)) <= this.MAX_VALID_IMPRESSION_YAW_ANGLE);
}

RawSessionDetection.prototype.setInitialDurations = function (detection) {
    if (this.initialDetectionDuration === undefined) this.initialDetectionDuration = detection.detectionDuration;
    if (this.initialDwellDuration === undefined) this.initialDwellDuration = detection.detectionDuration;
    if (this.initialAttentionDuration === undefined) this.initialAttentionDuration = detection.totalAttentionDuration;
}

RawSessionDetection.prototype.getDwellDuration = function (detection) {

    if (detection.currentAttentionDuration === 0) {
        if (this.isImpression(detection))
            return (detection.detectionDuration - this.initialDetectionDuration) - this.detectionDuration;
        else return 0;
    } else {
        return (detection.totalAttentionDuration - this.initialAttentionDuration) - this.attentionDuration;
    }

}

RawSessionDetection.prototype.setMaxSmiles = function (detection) {
    if (parseInt(detection.smile) > this.maxSmile) { this.maxSmile = parseInt(detection.smile) };
    if (parseInt(detection.viewing) && parseInt(detection.smile) > this.maxSmileViewing) { this.maxSmileViewing = parseInt(detection.smile) };
}

RawSessionDetection.prototype.parseGenderToHuminizedValue = function () { return ((this.gender < 0) ? 'Male' : (this.gender > 0) ? 'Female' : 'Uncertain'); }

RawSessionDetection.prototype.parseBooleanToHuminizedValue = function( value ) {
        
    if(value <= 0 ) return "No";
    else if(value > 0) return "Yes";
    return undefined;
    
} 

RawSessionDetection.prototype.parseMillisecondsToSeconds = function (value) {
    return (value > 0) ? (value / 1000) : 0.000;
}