'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 * Creates a Real-time Aggregated Data Object with totals and avg properties
 *  updated when _computeData is called from the raw data output in DeepSight Toolkit
 * @constructor
 */
function RealTimeAggregatedData() {
    this.count = 0;
    this.maleCount = 0;
    this.malePercentage = 0;
    this.femaleCount = 0;
    this.femalePercentage = 0;
    this.uncertainCount = 0;
    this.uncertainPercentage = 0;
    this.age = 0;
    this.smile = 0;
    this.withMask = 0;
    this.countViewer = 0;
    this.maleCountViewer = 0;
    this.malePercentageViewer = 0;
    this.femaleCountViewer = 0;
    this.femalePercentageViewer = 0;
    this.uncertainCountViewer = 0;
    this.uncertainPercentageViewer = 0;
    this.ageViewer = 0;
    this.smileViewer = 0;
    this.withMaskViewer = 0;
}

RealTimeAggregatedData.prototype._computeData = function (people, maskThreshold) {
    var self = this;
    try {

        self.count = people.length;

        var males, males_looking, females, females_looking, uncertains, uncertains_looking
            , total_age, total_age_looking, total_smile, total_smile_looking, people_is_looking
            , with_mask, with_mask_looking;
        males = males_looking = females = females_looking = uncertains = uncertains_looking
            = total_age = total_age_looking = total_smile = total_smile_looking = people_is_looking
            = with_mask = with_mask_looking = 0;

        if (self.count) {
            people.forEach(function (detection) {
                // Count totals gender
                if (detection.rawGender < 0) males++;
                else if (detection.rawGender > 0) females++;
                else uncertains++
                // Aggregate age and smile for avg
                total_age += detection.rawAge;
                total_smile += detection.rawSmile;
                if (detection.faceMask > maskThreshold) with_mask++;
                // Is Looking
                if (detection.viewing) {
                    people_is_looking++;
                    total_age_looking += detection.age;
                    total_smile_looking += detection.smile;

                    if (detection.rawGender < 0) males_looking++;
                    else if (detection.rawGender > 0) females_looking++;
                    else uncertains_looking++;

                    if (detection.faceMask > maskThreshold) with_mask_looking++;

                }

            });

            //Compute counter, avg and percentage 
            this.maleCount = males;
            this.femaleCount = females;
            this.uncertainCount = uncertains;
            this.malePercentage = parseInt(Math.floor((this.maleCount / this.count) * 100).toFixed(0));
            this.femalePercentage = parseInt(Math.floor((this.femaleCount / this.count) * 100).toFixed(0));
            this.uncertainPercentage = parseInt(Math.floor((this.uncertainCount / this.count) * 100).toFixed(0));
            this.age = parseInt((total_age / this.count).toFixed(0));
            this.smile = parseInt((total_smile / this.count).toFixed(0));
            this.withMask = with_mask;

            if (people_is_looking) {
                this.countViewer = people_is_looking;
                this.maleCountViewer = males_looking;
                this.femaleCountViewer = females_looking;
                this.uncertainCountViewer = uncertains_looking;
                this.malePercentageViewer = parseInt(Math.floor((this.maleCountViewer / this.countViewer) * 100).toFixed(0));
                this.femalePercentageViewer = parseInt(Math.floor((this.femaleCountViewer / this.countViewer) * 100).toFixed(0));
                this.uncertainPercentageViewer = parseInt(Math.floor((this.uncertainCountViewer / this.countViewer) * 100).toFixed(0));
                this.ageViewer = parseInt((total_age_looking / this.countViewer).toFixed(0));
                this.smileViewer = parseInt((total_smile_looking / this.countViewer).toFixed(0));
                this.withMaskViewer = with_mask_looking;
            }

        } else {
            // If not detections, set as 0 all values
            this._resetRealTimeAggregatedData();
        }

    } catch (error) {
        console.log("RealTimeAggregatedData.prototype._computeData : " + error);
    }
}


RealTimeAggregatedData.prototype._resetRealTimeAggregatedData = function () {
    this.count = 0;
    this.maleCount = 0;
    this.malePercentage = 0;
    this.femaleCount = 0;
    this.femalePercentage = 0;
    this.uncertainCount = 0;
    this.uncertainPercentage = 0;
    this.age = 0;
    this.smile = 0;
    this.withMask = 0;
    this.countViewer = 0;
    this.maleCountViewer = 0;
    this.malePercentageViewer = 0;
    this.femaleCountViewer = 0;
    this.femalePercentageViewer = 0;
    this.uncertainCountViewer = 0;
    this.uncertainPercentageViewer = 0;
    this.ageViewer = 0;
    this.smileViewer = 0;
    this.withMaskViewer = 0;
}